@@ -10,6 +10,7 @@
 package org.gnu.gconf;
 
 import org.gnu.glib.Boxed;
+import org.gnu.glib.Handle;
 
 /**
  * Stores an entry for a GConf "directory", including a key-value pair,
@@ -25,11 +26,11 @@ public class ConfEntry extends Boxed {
 	 * @param value
 	 */
 	public ConfEntry(String key, ConfValue value) {
-		handle = gconf_entry_new_nocopy(key, value.getHandle());
+		super(gconf_entry_new_nocopy(key, value.getHandle()));
 	}
 	
-	public ConfEntry(int handle) {
-		this.handle = handle;
+	public ConfEntry(Handle handle) {
+		super(handle);
 	}
 	
 	/**
@@ -45,7 +46,7 @@ public class ConfEntry extends Boxed {
 	 * @return
 	 */
 	public ConfValue getValue() {
-		int hndl = gconf_entry_get_value(handle);
+		Handle hndl = gconf_entry_get_value(handle);
 		return new ConfValue(hndl);
 	}
 	
@@ -58,7 +59,7 @@ public class ConfEntry extends Boxed {
 	 * @return
 	 */
 	public ConfValue stealValue() {
-		int hndl = gconf_entry_steal_value(handle);
+		Handle hndl = gconf_entry_steal_value(handle);
 		return new ConfValue(hndl);
 	}
 	
@@ -82,19 +83,15 @@ public class ConfEntry extends Boxed {
 		gconf_entry_set_is_default(handle, isDefault);
 	}
 
-	/****************************************
-	 * BEGINNING OF NATIVE CODE
-	 ****************************************/
-	native static final protected int gconf_entry_new_nocopy(String key, int value);
-	native static final protected String gconf_entry_get_key(int entry);
-	native static final protected int gconf_entry_get_value(int entry);
-	native static final protected int gconf_entry_steal_value(int entry);
-	native static final protected boolean gconf_entry_get_is_default(int entry);
-	native static final protected String gconf_entry_get_schema_name(int entry);
-	native static final protected void gconf_entry_set_is_default(int entry, boolean isDefault);
-	native static final protected void gconf_entry_set_schema_name(int entry, String schemaName);
-	native static final protected void gconf_entry_set_value_nocopy(int entry, int value);
-	/****************************************
-	 * END OF NATIVE CODE
-	 ****************************************/
+
+	native static final protected Handle gconf_entry_new_nocopy(String key, Handle value);
+	native static final protected String gconf_entry_get_key(Handle entry);
+	native static final protected Handle gconf_entry_get_value(Handle entry);
+	native static final protected Handle gconf_entry_steal_value(Handle entry);
+	native static final protected boolean gconf_entry_get_is_default(Handle entry);
+	native static final protected String gconf_entry_get_schema_name(Handle entry);
+	native static final protected void gconf_entry_set_is_default(Handle entry, boolean isDefault);
+	native static final protected void gconf_entry_set_schema_name(Handle entry, String schemaName);
+	native static final protected void gconf_entry_set_value_nocopy(Handle entry, Handle value);
+
 }
