@@ -10,6 +10,7 @@
 #include <jni.h>
 #include <gconf/gconf.h>
 #include <gconf/gconf-client.h>
+#include "jg_jnu.h"
 
 #ifndef _Included_org_gnu_gconf_ConfClient
 #define _Included_org_gnu_gconf_ConfClient
@@ -19,59 +20,56 @@ extern "C" {
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_get_default
- * Signature: ()I
  */
-JNIEXPORT jint JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1default
+JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1default
   (JNIEnv *env, jclass cls)
 {
-	return (jint)gconf_client_get_default();
+	return getHandleFromPointer(env, gconf_client_get_default());
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_get_for_engine
- * Signature: (I)I
  */
-JNIEXPORT jint JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1for_1engine
-  (JNIEnv *env, jclass cls, jint engine)
+JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1for_1engine
+  (JNIEnv *env, jclass cls, jobject engine)
 {
-	return (jint)gconf_client_get_for_engine((GConfEngine*)engine);
+	GConfEngine* engine_g = (GConfEngine*)getPointerFromHandle(env, engine);
+	return getHandleFromPointer(env, gconf_client_get_for_engine(engine_g));
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_add_dir
- * Signature: (ILjava/lang/String;I[I)V
  */
 JNIEXPORT void JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1add_1dir
-  (JNIEnv *env, jclass cls, jint client, jstring dir, jint preload, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring dir, jint preload, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, dir, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	gconf_client_add_dir((GConfClient*)client, str, (GConfClientPreloadType)preload, &err);
+	gconf_client_add_dir(client_g, str, (GConfClientPreloadType)preload, &err);
 	(*env)->ReleaseStringUTFChars(env, dir, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_remove_dir
- * Signature: (ILjava/lang/String;[I)V
  */
 JNIEXPORT void JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1remove_1dir
-  (JNIEnv *env, jclass cls, jint client, jstring dir, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring dir, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, dir, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	gconf_client_remove_dir((GConfClient*)client, str, &err);
+	gconf_client_remove_dir(client_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, dir, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL) 
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
 }
 
 typedef struct 
@@ -96,11 +94,11 @@ void notifyFree(gpointer data)
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_notify_add
- * Signature: (ILjava/lang/String;[I)I
  */
 JNIEXPORT jint JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1notify_1add
-  (JNIEnv *env, jobject obj, jint client, jstring namespace, jintArray exception)
+  (JNIEnv *env, jobject obj, jobject client, jstring namespace, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	static jmethodID methodID = NULL;
 	jclass cls = (*env)->GetObjectClass(env, obj);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
@@ -108,7 +106,7 @@ JNIEXPORT jint JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1notify_1add
 	NotifyData *data;
 	guint ret;
 	if (methodID == NULL) {
-		methodID = (*env)->GetMethodID(env, cls, "notifyCallback", "(ILjava/lang/String;)V");
+		methodID = (*env)->GetMethodID(env, cls, "notifyCallback", "(Lorg/gnu/glib/Handle;Ljava/lang/String;)V");
 		if (methodID == NULL)
 			return -1;
 	}
@@ -117,35 +115,34 @@ JNIEXPORT jint JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1notify_1add
 	data->obj = obj;
 	data->methodID = methodID;
 	data->namespace = namespace;
-	ret = gconf_client_notify_add((GConfClient*)client, str, notifyCallback, data, notifyFree, &err);
+	ret = gconf_client_notify_add(client_g, str, notifyCallback, data, notifyFree, &err);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
 	(*env)->ReleaseStringUTFChars(env, namespace, str);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
 	return ret;
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_notify_remove
- * Signature: (II)V
  */
 JNIEXPORT void JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1notify_1remove
-  (JNIEnv *env, jclass cls, jint client, jint cnxn)
+  (JNIEnv *env, jclass cls, jobject client, jint cnxn)
 {
-	gconf_client_notify_remove((GConfClient*)client, (int)cnxn);
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
+	gconf_client_notify_remove(client_g, (int)cnxn);
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_clear_cache
- * Signature: (I)V
  */
 JNIEXPORT void JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1clear_1cache
-  (JNIEnv *env, jclass cls, jint client)
+  (JNIEnv *env, jclass cls, jobject client)
 {
-	gconf_client_clear_cache((GConfClient*)client);
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
+	gconf_client_clear_cache(client_g);
 }
 
 /*
@@ -154,174 +151,159 @@ JNIEXPORT void JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1clear_1cache
  * Signature: (ILjava/lang/String;I[I)V
  */
 JNIEXPORT void JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1preload
-  (JNIEnv *env, jclass cls, jint client, jstring dir, jint type, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring dir, jint type, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, dir, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	gconf_client_preload((GConfClient*)client, str, (GConfClientPreloadType)type, &err);
+	gconf_client_preload(client_g, str, (GConfClientPreloadType)type, &err);
 	(*env)->ReleaseStringUTFChars(env, dir, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_set
- * Signature: (ILjava/lang/String;I[I)V
  */
 JNIEXPORT void JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set
-  (JNIEnv *env, jclass cls, jint client, jstring key, jint val, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jobject val, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
+	GConfValue* val_g = (GConfValue*)getPointerFromHandle(env, val);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	gconf_client_set((GConfClient*)client, str, (GConfValue*)val, &err);
+	gconf_client_set(client_g, str, val_g, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_get
- * Signature: (ILjava/lang/String;[I)I
  */
-JNIEXPORT jint JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get
-  (JNIEnv *env, jclass cls, jint client, jstring key, jintArray exception)
+JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	GConfValue* val = gconf_client_get((GConfClient*)client, str, &err);
+	GConfValue* val = gconf_client_get(client_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
-	return (jint)val;
+	return getHandleFromPointer(env, val);
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_get_without_default
- * Signature: (ILjava/lang/String;[I)I
  */
-JNIEXPORT jint JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1without_1default
-  (JNIEnv *env, jclass cls, jint client, jstring key, jintArray exception)
+JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1without_1default
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	GConfValue* val = gconf_client_get_without_default((GConfClient*)client, str, &err);
+	GConfValue* val = gconf_client_get_without_default(client_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
-	return (jint)val;
+	return getHandleFromPointer(env, val);
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_get_entry
- * Signature: (ILjava/lang/String;Ljava/lang/String;Z[I)I
  */
-JNIEXPORT jint JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1entry
-  (JNIEnv *env, jclass cls, jint client, jstring key, jstring locale, jboolean schemaDefault, jintArray exception)
+JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1entry
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jstring locale, jboolean schemaDefault, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str1 = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	gchar *str2 = (gchar*)(*env)->GetStringUTFChars(env, locale, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	GConfEntry* val = gconf_client_get_entry((GConfClient*)client, str1, str2, (gboolean)schemaDefault, &err);
+	GConfEntry* val = gconf_client_get_entry(client_g, str1, str2, (gboolean)schemaDefault, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str1);
 	(*env)->ReleaseStringUTFChars(env, locale, str2);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
-	return (jint)val;
+	return getHandleFromPointer(env, val);
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_get_default_from_schema
- * Signature: (ILjava/lang/String;[I)I
  */
-JNIEXPORT jint JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1default_1from_1schema
-  (JNIEnv *env, jclass cls, jint client, jstring key, jintArray exception)
+JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1default_1from_1schema
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	GConfValue* val = gconf_client_get_default_from_schema((GConfClient*)client, str, &err);
+	GConfValue* val = gconf_client_get_default_from_schema(client_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
-	return (jint)val;
+	return getHandleFromPointer(env, val);
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_unset
- * Signature: (ILjava/lang/String;[I)Z
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1unset
-  (JNIEnv *env, jclass cls, jint client, jstring key, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	gboolean val = gconf_client_unset((GConfClient*)client, str, &err);
+	gboolean val = gconf_client_unset(client_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
 	return (jboolean)val;
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_all_entries
- * Signature: (ILjava/lang/String;[I)[I
  */
-JNIEXPORT jintArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1all_1entries
-  (JNIEnv *env, jclass cls, jint client, jstring dir, jintArray exception)
+JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1all_1entries
+  (JNIEnv *env, jclass cls, jobject client, jstring dir, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, dir, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	GSList* val = gconf_client_all_entries((GConfClient*)client, str, &err);
-	int i = 0;
-	jintArray ar;
-	guint len;
+	GSList* val = gconf_client_all_entries(client_g, str, &err);
 	if (NULL == val)
 		return NULL;
-	len = g_slist_length(val);
-	ar = (*env)->NewIntArray(env, len);
-	for (i = 0; i < len; i++) {
-		GSList *item = g_slist_nth(val, i);
-		(*env)->SetIntArrayRegion(env, ar, i, 1, (jint*)item->data);
-	}
 	(*env)->ReleaseStringUTFChars(env, dir, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
-	return ar;
+	return getHandleArrayFromGSList(env, val);
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_all_dirs
- * Signature: (ILjava/lang/String;[I)jobjectArray
  */
 JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1all_1dirs
-  (JNIEnv *env, jclass cls, jint client, jstring dir, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring dir, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, dir, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	GSList* val = gconf_client_all_dirs((GConfClient*)client, str, &err);
+	GSList* val = gconf_client_all_dirs(client_g, str, &err);
 	int i = 0;
 	guint len;
 	jclass strCls;
@@ -340,150 +322,143 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1all_
 	}
 	(*env)->ReleaseStringUTFChars(env, dir, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
 	return ar;
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_suggest_sync
- * Signature: (I[I)V
  */
 JNIEXPORT void JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1suggest_1sync
-  (JNIEnv *env, jclass cls, jint client, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	gconf_client_suggest_sync((GConfClient*)client, &err);
+	gconf_client_suggest_sync(client_g, &err);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_dir_exists
- * Signature: (ILjava/lang/String;[I)Z
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1dir_1exists
-  (JNIEnv *env, jclass cls, jint client, jstring dir, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring dir, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, dir, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	gboolean val = gconf_client_dir_exists((GConfClient*)client, str, &err);
+	gboolean val = gconf_client_dir_exists(client_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, dir, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
 	return (jboolean)val;
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_get_float
- * Signature: (ILjava/lang/String;[I)D
  */
 JNIEXPORT jdouble JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1float
-  (JNIEnv *env, jclass cls, jint client, jstring key, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	gdouble val = gconf_client_get_float((GConfClient*)client, str, &err);
+	gdouble val = gconf_client_get_float(client_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
 	return (jdouble)val;
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_get_int
- * Signature: (ILjava/lang/String;[I)I
  */
 JNIEXPORT jint JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1int
-  (JNIEnv *env, jclass cls, jint client, jstring key, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	gint val = gconf_client_get_int((GConfClient*)client, str, &err);
+	gint val = gconf_client_get_int(client_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
 	return (jint)val;
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_get_string
- * Signature: (ILjava/lang/String;[I)Ljava/lang/String;
  */
 JNIEXPORT jstring JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1string
-  (JNIEnv *env, jclass cls, jint client, jstring key, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	GError *err = NULL;
-	gchar* val = gconf_client_get_string((GConfClient*)client, str, &err);
+	gchar* val = gconf_client_get_string(client_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
 	return (*env)->NewStringUTF(env, val);
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_get_bool
- * Signature: (ILjava/lang/String;[I)Z
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1bool
-  (JNIEnv *env, jclass cls, jint client, jstring key, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	gboolean val = gconf_client_get_bool((GConfClient*)client, str, &err);
+	gboolean val = gconf_client_get_bool(client_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
 	return (jboolean)val;
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_get_schema
- * Signature: (ILjava/lang/String;[I)I
  */
-JNIEXPORT jint JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1schema
-  (JNIEnv *env, jclass cls, jint client, jstring key, jintArray exception)
+JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1schema
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	GConfSchema* val = gconf_client_get_schema((GConfClient*)client, str, &err);
+	GConfSchema* val = gconf_client_get_schema(client_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL) 
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
-	return (jint)val;
+	return getHandleFromPointer(env, val);
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_get_list
- * Signature: (ILjava/lang/String;I[I)Ljava/lang/Object;
  */
 JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1list
-  (JNIEnv *env, jclass cls, jint client, jstring key, jint type, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jint type, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
 	GConfValueType vt = (GConfValueType)type;
-	GSList* val = gconf_client_get_list((GConfClient*)client, str, vt, &err);
+	GSList* val = gconf_client_get_list(client_g, str, vt, &err);
 	int i = 0;
 	jclass aCls = NULL;
 	guint len;
@@ -541,141 +516,135 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_
 	}
 	(*env)->ReleaseStringUTFChars(env, key, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
 	return ar;
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_get_pair
- * Signature: (ILjava/lang/String;II[I[I[I)Z
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1pair
-  (JNIEnv *env, jclass cls, jint client, jstring key, jint carType, jint cdrType, jintArray car, jintArray cdr, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jint carType, jint cdrType, jintArray car, jintArray cdr, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
 	gpointer carloc = (gpointer)(*env)->GetIntArrayElements(env, car, NULL);
 	gpointer cdrloc = (gpointer)(*env)->GetIntArrayElements(env, cdr, NULL);
-	gboolean val = gconf_client_get_pair((GConfClient*)client, str, (GConfValueType)carType, 
+	gboolean val = gconf_client_get_pair(client_g, str, (GConfValueType)carType, 
 			(GConfValueType)cdrType, carloc, cdrloc, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
 	(*env)->ReleaseIntArrayElements(env, car, (jint*)carloc, 0);
 	(*env)->ReleaseIntArrayElements(env, cdr, (jint*)cdrloc, 0);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
 	return (jboolean)val;
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_set_float
- * Signature: (ILjava/lang/String;D[I)Z
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1float
-  (JNIEnv *env, jclass cls, jint client, jstring key, jdouble val, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jdouble val, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	gboolean ret = gconf_client_set_float((GConfClient*)client, str, (gdouble)val, &err);
+	gboolean ret = gconf_client_set_float(client_g, str, (gdouble)val, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
 	return (jboolean)ret;
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_set_int
- * Signature: (ILjava/lang/String;I[I)Z
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1int
-  (JNIEnv *env, jclass cls, jint client, jstring key, jint val, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jint val, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	gboolean ret = gconf_client_set_int((GConfClient*)client, str, (gint)val, &err);
+	gboolean ret = gconf_client_set_int(client_g, str, (gint)val, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
 	return (jboolean)ret;
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_set_string
- * Signature: (ILjava/lang/String;Ljava/lang/String;[I)Z
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1string
-  (JNIEnv *env, jclass cls, jint client, jstring key, jstring val, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jstring val, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str1 = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	gchar *str2 = (gchar*)(*env)->GetStringUTFChars(env, val, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	gboolean ret = gconf_client_set_string((GConfClient*)client, str1, str2, &err);
+	gboolean ret = gconf_client_set_string(client_g, str1, str2, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str1);
 	(*env)->ReleaseStringUTFChars(env, key, str2);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
 	return (jboolean)ret;
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_set_bool
- * Signature: (ILjava/lang/String;Z[I)Z
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1bool
-  (JNIEnv *env, jclass cls, jint client, jstring key, jboolean val, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jboolean val, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	gboolean ret = gconf_client_set_bool((GConfClient*)client, str, (gboolean)val, &err);
+	gboolean ret = gconf_client_set_bool(client_g, str, (gboolean)val, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
 	return (jboolean)ret;
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_set_schema
- * Signature: (ILjava/lang/String;I[I)Z
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1schema
-  (JNIEnv *env, jclass cls, jint client, jstring key, jint val, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jobject val, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
+	GConfSchema* val_g = (GConfSchema*)getPointerFromHandle(env, val);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	gboolean ret = gconf_client_set_schema((GConfClient*)client, str, (GConfSchema*)val, &err);
+	gboolean ret = gconf_client_set_schema(client_g, str, val_g, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
 	return (jboolean)ret;
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_set_list
- * Signature: (ILjava/lang/String;I[Ljava/lang/Object;[I)Z
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1list
-  (JNIEnv *env, jclass cls, jint client, jstring key, jint type, jobjectArray list, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jint type, jobjectArray list, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
 	GConfValueType vt = (GConfValueType)type;
@@ -736,45 +705,44 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1lis
 		}
 		gslist = g_slist_append(gslist, data);
 	}
-	ret = gconf_client_set_list((GConfClient*)client, str, vt, gslist, &err);
+	ret = gconf_client_set_list(client_g, str, vt, gslist, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
 	return (jboolean)ret;
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_set_pair
- * Signature: (ILjava/lang/String;IIII[I)Z
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1pair
-  (JNIEnv *env, jclass cls, jint client, jstring key, jint carType, jint cdrType, jint car, jint cdr, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jint carType, jint cdrType, jint car, jint cdr, jintArray exception)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	gboolean ret = gconf_client_set_pair((GConfClient*)client, str, (GConfValueType)carType, 
+	gboolean ret = gconf_client_set_pair(client_g, str, (GConfValueType)carType, 
 			(GConfValueType)cdrType, (gconstpointer)car, (gconstpointer)cdr, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
 	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) {
+	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	}
 	return (jboolean)ret;
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_value_changed
- * Signature: (ILjava/lang/String;I)V
  */
 JNIEXPORT void JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1value_1changed
-  (JNIEnv *env, jclass cls, jint client, jstring key, jint value)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jobject value)
 {
+	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	gconf_client_value_changed((GConfClient*)client, str, (GConfValue*)value);
+	GConfValue* value_g = (GConfValue*)getPointerFromHandle(env, value);
+	gconf_client_value_changed(client_g, str, value_g);
 	(*env)->ReleaseStringUTFChars(env, key, str);
 }
 
